@@ -3,7 +3,7 @@
  * buildargv.c
  *	  a routines for parsing string to argc, argv format
  *
- * Portions Copyright (c) 2017-2022 Pavel Stehule
+ * Portions Copyright (c) 2017-2023 Pavel Stehule
  *
  * IDENTIFICATION
  *	  src/buildargv.c
@@ -245,8 +245,12 @@ static void
 print_info(void)
 {
 
+#ifdef HAVE_SYS_UTSNAME_H
+
 	struct utsname u_name;
 
+#endif
+
 	fprintf(stdout, "pspg-%s\n", PSPG_VERSION);
 
 #ifdef HAVE_LIBREADLINE
@@ -302,7 +306,7 @@ print_info(void)
 
 #else
 
-	fprintf(stdout, "without ncurses extended function support\n")
+	fprintf(stdout, "without ncurses extended function support\n");
 
 #endif
 
@@ -316,6 +320,11 @@ print_info(void)
 
 #endif
 
+#ifdef PDCURSES
+
+	fprintf(stdout, "with pdcurses %s\n", PDC_VERDOT);
+
+#endif
 
 #ifdef HAVE_POSTGRESQL
 
