@@ -4,7 +4,7 @@
  *	  a routines for implementation (parsing, execution) of backslash
  *	  commands
  *
- * Portions Copyright (c) 2017-2022 Pavel Stehule
+ * Portions Copyright (c) 2017-2023 Pavel Stehule
  *
  * IDENTIFICATION
  *	  src/bscommands.c
@@ -148,7 +148,7 @@ parse_exported_spec(const char *instr,
 	const char   *token;
 	int		n;
 
-	spec->command = cmd_CopyAllLines;
+	spec->command = cmd_Copy;
 	spec->format = CLIPBOARD_FORMAT_TEXT;
 	spec->rows = 0;
 	spec->percent = 0.0;
