@@ -3,13 +3,15 @@
  * events.c
  *	  handles all events - tty and content
  *
- * Portions Copyright (c) 2017-2022 Pavel Stehule
+ * Portions Copyright (c) 2017-2023 Pavel Stehule
  *
  * IDENTIFICATION
  *	  src/inputs.c
  *
  *-------------------------------------------------------------------------
  */
+#define PDC_NCMOUSE
+
 #include <errno.h>
 #include <fcntl.h>
 #include <libgen.h>
@@ -68,6 +70,12 @@ static bool saved_event_is_valid = false;
 static bool close_f_tty = false;
 static bool close_f_data = false;
 
+#ifdef PDCURSES
+
+static bool is_button1_pressed = false;
+
+#endif
+
 int		pspg_esc_delay;
 
 
@@ -77,17 +85,138 @@ int		pspg_esc_delay;
  *************************************
  */
 
+#ifdef  PDCURSES
+
+static bool
+is_alt(int *keycode)
+{
+	switch (*keycode)
+	{
+		case ALT_0:
+			*keycode = '0';
+			return true;
+		case ALT_1:
+			*keycode = '1';
+			return true;
+		case ALT_2:
+			*keycode = '2';
+			return true;
+		case ALT_3:
+			*keycode = '3';
+			return true;
+		case ALT_4:
+			*keycode = '4';
+			return true;
+		case ALT_5:
+			*keycode = '5';
+			return true;
+		case ALT_6:
+			*keycode = '6';
+			return true;
+		case ALT_7:
+			*keycode = '7';
+			return true;
+		case ALT_8:
+			*keycode = '8';
+			return true;
+		case ALT_9:
+			*keycode = '9';
+			return true;
+		case ALT_A:
+			*keycode = 'a';
+			return true;
+		case ALT_B:
+			*keycode = 'b';
+			return true;
+		case ALT_C:
+			*keycode = 'c';
+			return true;
+		case ALT_D:
+			*keycode = 'd';
+			return true;
+		case ALT_E:
+			*keycode = 'e';
+			return true;
+		case ALT_F:
+			*keycode = 'f';
+			return true;
+		case ALT_G:
+			*keycode = 'g';
+			return true;
+		case ALT_H:
+			*keycode = 'h';
+			return true;
+		case ALT_I:
+			*keycode = 'i';
+			return true;
+		case ALT_J:
+			*keycode = 'j';
+			return true;
+		case ALT_K:
+			*keycode = 'k';
+			return true;
+		case ALT_L:
+			*keycode = 'l';
+			return true;
+		case ALT_M:
+			*keycode = 'm';
+			return true;
+		case ALT_N:
+			*keycode = 'n';
+			return true;
+		case ALT_O:
+			*keycode = 'o';
+			return true;
+		case ALT_P:
+			*keycode = 'p';
+			return true;
+		case ALT_Q:
+			*keycode = 'q';
+			return true;
+		case ALT_R:
+			*keycode = 'r';
+			return true;
+		case ALT_S:
+			*keycode = 's';
+			return true;
+		case ALT_T:
+			*keycode = 't';
+			return true;
+		case ALT_U:
+			*keycode = 'u';
+			return true;
+		case ALT_V:
+			*keycode = 'v';
+			return true;
+		case ALT_W:
+			*keycode = 'w';
+			return true;
+		case ALT_X:
+			*keycode = 'x';
+			return true;
+		case ALT_Y:
+			*keycode = 'y';
+			return true;
+		case ALT_Z:
+			*keycode = 'x';
+			return true;
+
+		default:
+			return false;
+	}
+}
+
+#endif
+
 /*
  * Read one ncurses event
  */
-static int
-get_ncurses_event(NCursesEventData *nced, bool *sigint)
+static bool
+get_ncurses_event(NCursesEventData *nced, bool *sigint, bool *sigwinch)
 {
 	bool	first_event = true;
 	int		ok = true;
 
-	*sigint = false;
-
 #if NCURSES_WIDECHAR > 0 && defined HAVE_NCURSESW
 
 	wint_t	ch;
@@ -95,6 +224,13 @@ get_ncurses_event(NCursesEventData *nced, bool *sigint)
 
 #endif
 
+	*sigint = false;
+	*sigwinch = false;
+
+	nced->keycode = PSPG_NOTASSIGNED_CODE;
+	nced->ignore_it = false;
+	nced->alt = false;
+
 	/*
 	 * When ALT key is used, then ncurses generates two keycodes. And then
 	 * we have to read input 2x times.
@@ -116,11 +252,73 @@ repeat:
 
 #endif
 
+#ifdef DEBUG_PIPE
+
+	fprintf(debug_pipe, "*** keycode: %d, err: %d\n", nced->keycode, errno);
+
+#endif
+
+	if (errno == EINTR)
+	{
+		if (handle_sigint)
+		{
+			*sigint = true;
+			handle_sigint = false;
+		}
+
+		if (handle_sigwinch)
+		{
+			*sigwinch = true;
+			handle_sigwinch = false;
+		}
+
+		return false;
+	}
+
 	if (errno == 0)
 	{
+		if (nced->keycode == ERR)
+		{
+				nced->keycode = PSPG_NOTASSIGNED_CODE;
+				nced->alt = false;
+				nced->ignore_it = true;
+				return true;
+		}
+
 		if (nced->keycode == KEY_MOUSE)
 		{
 			ok = getmouse(&nced->mevent) == OK;
+
+#ifdef PDCURSES
+
+			/*
+			 * event filter - we want to report mouse move only when
+			 * BUTTON1 is pressed (note: we have not xterm mouse mode 1002
+			 * there, because terminalinfo is just fake API there.
+			 */
+			if (nced->mevent.bstate & BUTTON1_PRESSED)
+			{
+				is_button1_pressed = true;
+			}
+			else if (nced->mevent.bstate & BUTTON1_RELEASED)
+			{
+				is_button1_pressed = false;
+			}
+
+			if (nced->mevent.bstate & REPORT_MOUSE_POSITION)
+			{
+				if (is_button1_pressed)
+				{
+						return true;
+				}
+
+				nced->keycode = PSPG_NOTASSIGNED_CODE;
+				nced->ignore_it = true;
+				return true;
+			}
+
+#endif
+
 		}
 		else if (nced->keycode == PSPG_ESC_CODE) /* Escape (before ALT chars) */
 		{
@@ -132,24 +330,6 @@ repeat:
 		}
 	}
 
-	if ((nced->keycode == ERR && errno == EINTR) ||
-		 handle_sigint || handle_sigwinch)
-	{
-		if (handle_sigwinch)
-		{
-			nced->keycode = KEY_RESIZE;
-			return true;
-		}
-		else if (handle_sigint)
-		{
-			*sigint = true;
-			handle_sigint = false;
-			return false;
-		}
-
-		return false;
-	}
-
 	/*
 	 * Workaround for issue #204. MacOS returns ERR when there are not
 	 * any other activity after ESC in ESCDELAY limit.
@@ -157,8 +337,16 @@ repeat:
 	if (nced->keycode == ERR && !first_event && errno == 0)
 		nced->keycode = PSPG_NOTASSIGNED_CODE;
 
+#if PDCURSES
+
+	nced->alt = is_alt(&nced->keycode);
+
+#else
+
 	nced->alt = !first_event;
 
+#endif
+
 	return ok;
 }
 
@@ -173,6 +361,7 @@ _get_pspg_event(NCursesEventData *nced,
 			    int timeout)
 {
 	bool	sigint;
+	bool	sigwinch;
 	bool	first_event = true;
 	bool	first_loop = true;
 	bool	without_timeout = timeout == -1;
@@ -203,9 +392,7 @@ _get_pspg_event(NCursesEventData *nced,
 	else if (handle_sigwinch)
 	{
 		handle_sigwinch = false;
-		nced->alt = false;
-		nced->keycode = KEY_RESIZE;
-		return PSPG_NCURSES_EVENT;
+		return PSPG_SIGWINCH_EVENT;
 	}
 
 	/*
@@ -215,7 +402,7 @@ _get_pspg_event(NCursesEventData *nced,
 	 */
 	if (only_tty_events && zero_timeout)
 	{
-		if (get_ncurses_event(nced, &sigint))
+		if (get_ncurses_event(nced, &sigint, &sigwinch))
 			return PSPG_NCURSES_EVENT;
 		else if (sigint)
 		{
@@ -227,6 +414,10 @@ _get_pspg_event(NCursesEventData *nced,
 
 			return PSPG_SIGINT_EVENT;
 		}
+		else if (sigwinch)
+		{
+			return PSPG_SIGWINCH_EVENT;
+		}
 		else
 			return PSPG_NOTHING_VALID_EVENT;
 	}
@@ -329,18 +520,38 @@ repeat_reading:
 			else if (handle_sigwinch)
 			{
 				handle_sigwinch = false;
-				nced->alt = false;
-				nced->keycode = KEY_RESIZE;
-				return PSPG_NCURSES_EVENT;
+				return PSPG_SIGWINCH_EVENT;
 			}
 
-			log_row("poll error (%s)", strerror(errno));
+#ifdef PDCURSES
+
+			/*
+			 * I didn't find how to resize term in pdcurses without crash.
+			 * So instead to call resize_term from custom space, use pdcurses
+			 * sigwinch handler, and force pdcurses to handle resizing after
+			 * terminal is resized.
+			 */
+			else if (is_termresized())
+			{
+				goto ncurses_get;
+			}
+
+#endif
+
+			log_row("poll error (%s) %d", strerror(errno));
 		}
 		else if (poll_num > 0)
 		{
 			if (fds[0].revents)
 			{
-				if (get_ncurses_event(nced, &sigint))
+
+#ifdef PDCURSES
+
+ncurses_get:
+
+#endif
+
+				if (get_ncurses_event(nced, &sigint, &sigwinch))
 				{
 					if (nced->alt && nced->keycode == PSPG_NOTASSIGNED_CODE && first_event && timeout != 0)
 					{
@@ -386,6 +597,10 @@ repeat_reading:
 					if (!only_tty_events)
 						return PSPG_SIGINT_EVENT;
 				}
+				else if (sigwinch)
+				{
+					return PSPG_SIGWINCH_EVENT;
+				}
 			}
 			else if (fds[1].revents)
 			{
@@ -545,6 +760,9 @@ get_pspg_event(NCursesEventData *nced,
 		case PSPG_SIGINT_EVENT:
 			event_name = "SIGINT";
 			break;
+		case PSPG_SIGWINCH_EVENT:
+			event_name = "SIGWINCH";
+			break;
 		case PSPG_FATAL_EVENT:
 			event_name = "FATAL";
 			break;
@@ -563,16 +781,21 @@ get_pspg_event(NCursesEventData *nced,
 	{
 		char	buffer[20];
 
-		if (nced->keycode == KEY_MOUSE)
-			sprintf(buffer, ", bstate: %08lx", (unsigned long) nced->mevent.bstate);
-		else
-			buffer[0] = '\0';
+		if (!nced->ignore_it)
+		{
+			if (nced->keycode == KEY_MOUSE)
+				sprintf(buffer, ", bstate: %08lx", (unsigned long) nced->mevent.bstate);
+			else
+				buffer[0] = '\0';
 
-		fprintf(debug_pipe, "*** ncurses event %s%s%s (%d) ***\n",
-			  nced->alt ? "Alt " : "",
-			  keyname(nced->keycode),
-			  buffer,
-			  nced->keycode);
+			fprintf(debug_pipe, "*** ncurses event %s%s%s (%d) ***\n",
+				  nced->alt ? "Alt " : "",
+				  nced->keycode ? keyname(nced->keycode) : "0",
+				  buffer,
+				  nced->keycode);
+		}
+		else
+			fprintf(debug_pipe, "*** ignored ncurses event ****\n");
 	}
 
 	fflush(debug_pipe);
