@@ -3,7 +3,7 @@
  * pretty-csv.c
  *	  import and formatting csv and tsv documents
  *
- * Portions Copyright (c) 2017-2022 Pavel Stehule
+ * Portions Copyright (c) 2017-2023 Pavel Stehule
  *
  * IDENTIFICATION
  *	  src/pretty-csv.c
@@ -66,6 +66,9 @@ typedef struct
 	bool		ignore_short_rows;
 } PrintConfigType;
 
+static void pb_putc_repeat(PrintbufType *printbuf, int n, int c);
+
+
 /*
  * Add new row to LineBuffer
  */
@@ -103,19 +106,62 @@ pb_flush_line(PrintbufType *printbuf)
 static void
 pb_write(PrintbufType *printbuf, const char *str, int size)
 {
-	if (printbuf->free < size)
+	int		curr_dspl_width = 0;
+
+	while (size)
 	{
-		printbuf->size += 10 * 1024;
-		printbuf->free += 10 * 1024;
+		int		charsize, bytes;
+		bool	tabsubst = false;
 
-		printbuf->buffer = realloc(printbuf->buffer, printbuf->size);
-		if (!printbuf->buffer)
-			leave("out of memory while serialize csv output");
+		charsize = use_utf8 ? utf8charlen(*str) : 1;
+
+		if (charsize == 1 && *str == '\t')
+		{
+			bytes = 0;
+			tabsubst = true;
+
+			do
+			{
+				curr_dspl_width++;
+				bytes++;
+			} while (curr_dspl_width % 8 != 0);
+		}
+		else if (use_utf8)
+		{
+			curr_dspl_width += utf_dsplen(str);
+			bytes = charsize;
+		}
+		else
+		{
+			curr_dspl_width += *str >= 0x20 ? 1 : 0;
+			bytes = charsize;
+		}
+
+		if (bytes > printbuf->free)
+		{
+			printbuf->size += 10 * 1024;
+
+			printbuf->buffer = realloc(printbuf->buffer, printbuf->size);
+			if (!printbuf->buffer)
+				leave("out of memory while serialize csv output");
+
+			printbuf->free = printbuf->size - printbuf->used;
+		}
+
+		if (tabsubst)
+		{
+			pb_putc_repeat(printbuf, bytes, ' ');
+		}
+		else
+		{
+			memcpy(printbuf->buffer + printbuf->used, str, bytes);
+			printbuf->used += bytes;
+			printbuf->free -= bytes;
+		}
+
+		str += charsize;
+		size -= charsize;
 	}
-
-	memcpy(printbuf->buffer + printbuf->used, str, size);
-	printbuf->used += size;
-	printbuf->free -= size;
 }
 
 static void
@@ -124,16 +170,14 @@ pb_writes(PrintbufType *printbuf, const char *str)
 	pb_write(printbuf, str, strlen(str));
 }
 
-
 static void
 pb_write_repeat(PrintbufType *printbuf, int n, const char *str, int size)
 {
 	bool	need_realloc = false;
 
-	while (printbuf->free < size * n)
+	while (printbuf->free < (size * n))
 	{
 		printbuf->size += 10 * 1024;
-		printbuf->free += 10 * 1024;
 
 		need_realloc = true;
 	}
@@ -141,6 +185,8 @@ pb_write_repeat(PrintbufType *printbuf, int n, const char *str, int size)
 	if (need_realloc)
 	{
 		printbuf->buffer = realloc(printbuf->buffer, printbuf->size);
+		printbuf->free = printbuf->size - printbuf->used;
+
 		if (!printbuf->buffer)
 			leave("out of memory while serialize csv output");
 	}
@@ -191,14 +237,13 @@ pb_putc_repeat(PrintbufType *printbuf, int n, int c)
 	while (printbuf->free < n)
 	{
 		printbuf->size += 10 * 1024;
-		printbuf->free += 10 * 1024;
-
 		need_realloc = true;
 	}
 
 	if (need_realloc)
 	{
 		printbuf->buffer = realloc(printbuf->buffer, printbuf->size);
+		printbuf->free = printbuf->size - printbuf->used;
 		if (!printbuf->buffer)
 			leave("out of memory while serialize csv output");
 	}
@@ -391,7 +436,7 @@ pb_put_line(char *str, bool multiline, PrintbufType *printbuf)
 		pb_write(printbuf, str, size);
 	}
 	else
-		pb_puts(printbuf, str);
+		pb_write(printbuf, str, strlen(str));
 
 	return nextline;
 }
@@ -499,23 +544,34 @@ pb_print_rowbuckets(PrintbufType *printbuf,
 
 						if (!use_utf8)
 						{
-							if (multiline)
-							{
-								char   *ptr = field;
-								width = 0;
+							char	   *ptr = field;
 
-								while (*ptr)
+							width = 0;
+
+							while (*ptr)
+							{
+								if (*ptr == '\n')
 								{
-									if (*ptr++ == '\n')
+									more_lines = true;
+									ptr += 1;
+									break;
+								}
+								else if (*ptr == '\t')
+								{
+									do
 									{
-										more_lines |= true;
-										break;
-									}
-									width += 1;
+										width++;
+									} while (width % 8 != 0);
+									ptr += 1;
+								}
+								else
+								{
+									if (*ptr >= 0x20)
+										width++;
+
+									ptr += 1;
 								}
 							}
-							else
-								width = strlen(field);
 						}
 						else
 						{
@@ -740,10 +796,11 @@ postprocess_fields(int nfields,
 
 		if (!use_utf8)
 		{
-			size_t		cw = 0;
+			size_t		max_width;
 			char	    *ptr = row->fields[i];
 
 			width = 0;
+			max_width = 0;
 
 			while (*ptr)
 			{
@@ -752,17 +809,28 @@ postprocess_fields(int nfields,
 				else if (*ptr != '-' && *ptr != ' ' && *ptr != ':')
 					total += 1;
 
-				if (*ptr++ == '\n')
+				if (*ptr == '\n')
 				{
 					multiline = true;
-					width = cw > width ? cw : width;
-					cw = 0;
+					max_width = width > max_width ? width : max_width;
+					width = 0;
 				}
-				else
-					cw++;
+				else if (*ptr == '\t')
+				{
+					do
+					{
+						width++;
+					} while (width % 8 != 0);
+				}
+				else if (*ptr >= 0x20)
+				{
+					width++;
+				}
+
+				ptr += 1;
 			}
 
-			width = cw > width ? cw : width;
+			width = width > max_width ? width : max_width;
 		}
 		else
 			width = utf_string_dsplen_multiline(row->fields[i],
@@ -1136,7 +1204,7 @@ read_csv(RowBucketType *rb,
 
 			if (skip_initial)
 			{
-				if (c == ' ')
+				if (c == ' ' || c == '\t')
 					goto next_char;
 
 				skip_initial = false;
@@ -1215,7 +1283,7 @@ read_csv(RowBucketType *rb,
 				found_string = false;
 				first_nw = pos;
 			}
-			else if (instr || c != ' ')
+			else if (instr || (c != ' ' && c != '\t'))
 			{
 				last_nw = pos;
 			}
