@@ -3,7 +3,7 @@
  * print.c
  *	  visualisation loaded data
  *
- * Portions Copyright (c) 2017-2022 Pavel Stehule
+ * Portions Copyright (c) 2017-2023 Pavel Stehule
  *
  * IDENTIFICATION
  *	  src/print.c
@@ -42,6 +42,7 @@
 #include "unicode.h"
 
 #include <ctype.h>
+#include <limits.h>
 
 #ifndef A_ITALIC
 #define A_ITALIC	A_DIM
@@ -247,7 +248,7 @@ print_column_names(WINDOW *win,
 	/* position starts from zero again to be comparable with maxx */
 	pos -= srcx;
 
-	if (selected_xmin != -1)
+	if (selected_xmin != INT_MIN)
 	{
 		selected_xmin -= srcx;
 		selected_xmax -= srcx;
@@ -263,7 +264,7 @@ print_column_names(WINDOW *win,
 		bool	is_cursor = vcursor_xmin <= pos && pos <= vcursor_xmax;
 		bool	is_in_range = false;
 
-		is_in_range = selected_xmin != -1 && pos != -1 &&
+		is_in_range = selected_xmin != INT_MIN && pos != -1 &&
 						  pos >= selected_xmin && pos <= selected_xmax;
 
 		bytes = charlen(ptr);
@@ -351,7 +352,7 @@ print_column_names(WINDOW *win,
 		act_width = act_xmax - act_xmin + 1;
 
 		is_cursor = vcursor_xmin <= act_xmin && act_xmin <= vcursor_xmax;
-		is_in_range = selected_xmin != -1 &&
+		is_in_range = selected_xmin != INT_MIN &&
 						  act_xmin >= selected_xmin && act_xmin <= selected_xmax;
 
 		if (is_in_range)
@@ -700,14 +701,18 @@ parse_line(char *line, SpecialWord *words, int maxwords)
 	int		nwords = 0;
 	int		pos = 0;
 	bool	first_nonspace = true;
+	char   *first_char = line;
 
 	/*
 	 * When text starts on line start and first char is upper
 	 * char, then mark this text until double colon.
 	 *
 	 * like "Usage:"
+	 *
+	 * Exceptions: sentences (ending by . or contains \ )
 	 */
-	if (is_upper_char(line) || strncmp(line, "psql", 4) == 0)
+	if ((is_upper_char(line) || strncmp(line, "psql", 4) == 0) &&
+		!strchr(line, '.') && !strchr(line, '\\'))
 	{
 		char   *aux_line = line;
 
@@ -757,6 +762,15 @@ parse_line(char *line, SpecialWord *words, int maxwords)
 		/* psql's shell options */
 		else if (*line == '-')
 		{
+			/* when dash is inside world, then it is not an option */
+			if (line > first_char && is_ascii_alnum(line[-1]))
+			{
+				line += 1;
+				pos += 1;
+
+				continue;
+			}
+
 			words[nwords].start_pos = pos;
 			words[nwords].typ = 1;
 
@@ -769,10 +783,10 @@ parse_line(char *line, SpecialWord *words, int maxwords)
 			if (pos - words[nwords].start_pos > 2)
 				continue;
 
-			if (!is_ascii_alnum(*line))
+			if (!(is_ascii_alnum(*line) || *line == '?' || *line == '!'))
 				continue;
 
-			while (is_ascii_alnum(*line) || *line == '-')
+			while (is_ascii_alnum(*line) || *line == '-' || *line == '?' || *line == '!')
 			{
 				line += 1;
 				pos += 1;
@@ -1138,11 +1152,11 @@ window_fill(int window_identifier,
 			 */
 			if (is_fix_rows_only && rowstr == desc->namesline )
 			{
-				int		loc_selected_xmin = -1;
-				int		loc_selected_xmax = -1;
+				int		loc_selected_xmin = INT_MIN;
+				int		loc_selected_xmax = INT_MIN;
 
 				/* mark columns names only when columns are selected */
-				if (selected_xmin != -1 && scrdesc->selected_first_row == -1)
+				if (selected_xmin != INT_MIN && scrdesc->selected_first_row == -1)
 				{
 					loc_selected_xmin = selected_xmin;
 					loc_selected_xmax = selected_xmax;
@@ -1208,7 +1222,7 @@ window_fill(int window_identifier,
 				is_selected_row = rowno >= scrdesc->selected_first_row + 1 &&
 								  rowno < scrdesc->selected_first_row + 1 + scrdesc->selected_rows;
 
-				is_selected_columns = is_selectable && selected_xmin != -1;
+				is_selected_columns = is_selectable && selected_xmin != INT_MIN;
 
 				while (i < maxx)
 				{
@@ -1239,7 +1253,7 @@ window_fill(int window_identifier,
 					{
 						if (is_selected_row)
 						{
-							if (selected_xmin != -1 && pos != -1)
+							if (selected_xmin != INT_MIN && pos != -1)
 							{
 								if (pos >= selected_xmin && pos <= selected_xmax)
 									is_in_range = true;
