@@ -3,7 +3,7 @@
  * unicode.c
  *	  unicode and wide chars routines
  *
- * Portions Copyright (c) 2017-2022 Pavel Stehule
+ * Portions Copyright (c) 2017-2023 Pavel Stehule
  *
  * IDENTIFICATION
  *	  src/unicode.c
@@ -284,7 +284,17 @@ utf_string_dsplen(const char *s, int bytes)
 	{
 		const char c = *s;
 
-		if (c >= 0x20 && c < 0x7f)
+		if (c == '\t')
+		{
+			do
+			{
+				result++;
+			} while (result % 8 != 0);
+
+			s += 1;
+			bytes -= 1;
+		}
+		else if (c >= 0x20 && c < 0x7f)
 		{
 			s += 1;
 			result += 1;
@@ -344,7 +354,17 @@ utf_string_dsplen_multiline(const char *s, size_t max_bytes, bool *multiline, bo
 		}
 
 		clen = utf8charlen(*ptr);
-		rowlen += utf_dsplen(ptr);
+		if (clen == 1 && *ptr == '\t')
+		{
+			/* this code is designed like pg_wcssize */
+			do
+			{
+				rowlen++;
+			} while (rowlen % 8 != 0);
+		}
+		else
+			rowlen += utf_dsplen(ptr);
+
 		ptr += clen;
 		max_bytes -= clen;
 	}
